import adiffParser from 'osm-adiff-parser-saxjs';
import jsonParser from 'real-changesets-parser';
import moment from 'moment';
import React from 'react';
import mapboxgl from 'mapbox-gl';
import { render as render$1 } from 'react-dom';
import { featureCollection } from '@turf/helpers';
import bbox from '@turf/bbox';
import turfBboxPolygon from '@turf/bbox-polygon';
import PropTypes from 'prop-types';
import onClickOutside from 'react-click-outside';

const config = {
  overpassBase: 'https://overpass.maptime.in/api/interpreter',
  osmBase: 'https://www.openstreetmap.org/api/0.6/',
  mapboxAccessToken: 'pk.eyJ1Ijoib3BlbnN0cmVldG1hcCIsImEiOiJjam10OXpmc2YwMXI5M3BqeTRiMDBqMHVyIn0.LIcIDe3TZLSDdTWDoojzNg',
  S3_URL: 'https://s3.amazonaws.com/mapbox/real-changesets/production/'
};

function query(changesetID) {
  var url = config.osmBase + 'changeset/' + changesetID;
  var options = {
    'Response-Type': 'document'
  };
  return fetch(url, options).then(r => r.text()).then(r => {
    const parser = new DOMParser();
    let xml;

    try {
      xml = parser.parseFromString(r, 'text/xml');
    } catch (e) {
      throw e;
    }

    var csFeature = xml.getElementsByTagName('changeset')[0];
    var cs = csFeature.attributes;
    var uid = cs.uid.textContent;
    var user = cs.user.textContent;
    var from = moment(cs.created_at.textContent, 'YYYY-MM-DDTHH:mm:ss\\Z').subtract(1, 'seconds').format('YYYY-MM-DDTHH:mm:ss\\Z');
    var to = cs.closed_at ? cs.closed_at.textContent : null;
    var left = cs.min_lon ? cs.min_lon.textContent : -180;
    var bottom = cs.min_lat ? cs.min_lat.textContent : -90;
    var right = cs.max_lon ? cs.max_lon.textContent : 180;
    var top = cs.max_lat ? cs.max_lat.textContent : 90;
    return {
      id: changesetID,
      uid: uid,
      user: user,
      from: from,
      to: to,
      bbox: {
        left: left,
        bottom: bottom,
        right: right,
        top: top
      }
    };
  });
}

function getChangeset(changesetID, overpassBase = config.overpassBase) {
  return query(changesetID).then(changeset => {
    var url = config.S3_URL + changesetID + '.json';
    return fetch(url).then(r => {
      if (r.ok) return r.json(); // Fallback to overpass

      return Promise.reject();
    }).then(r => {
      var geojson = jsonParser(r);
      var featureMap = getFeatureMap(geojson);
      var ret = {
        geojson: geojson,
        featureMap: featureMap,
        changeset: changeset
      };
      return ret;
    }).catch(() => fetchFromOverPass(changesetID, changeset, overpassBase));
  });
}

function fetchFromOverPass(changesetID, changeset, overpassBase) {
  var data = getDataParam(changeset);
  var bbox = getBboxParam(changeset.bbox);
  var url = overpassBase + '?data=' + data + '&bbox=' + bbox;
  return fetch(url, {
    'Response-Type': 'application/osm3s+xml'
  }).then(r => r.text()).then(response => {
    return new Promise((res, rej) => {
      adiffParser(response, null, (err, json) => {
        if (err) {
          return rej({
            msg: 'Failed to parser adiff xml.',
            error: err
          });
        }

        var elements = Object.keys(json).reduce((result, item) => result.concat(json[item]), []);
        var geojson = jsonParser({
          elements: elements
        });
        var featureMap = getFeatureMap(geojson);
        var ret = {
          geojson: geojson,
          featureMap: featureMap,
          changeset: changeset
        };
        return res(ret);
      });
    });
  }).catch(err => Promise.reject({
    msg: 'Overpass query failed.',
    error: err
  }));
}

function getDataParam(c) {
  return '[out:xml][adiff:%22' + c.from.toString() + ',%22,%22' + c.to.toString() + '%22];(node(bbox)(changed);way(bbox)(changed);relation(bbox)(changed););out%20meta%20geom(bbox);';
}

function getBboxParam(bbox) {
  return [bbox.left, bbox.bottom, bbox.right, bbox.top].join(',');
}

function getFeatureMap(geojson) {
  var features = geojson.features;
  var featureMap = {};

  for (var i = 0, len = features.length; i < len; i++) {
    var id = features[i].properties.id;
    featureMap[id] = featureMap[id] || [];
    featureMap[id].push(features[i]);
  }

  return featureMap;
}

function propsDiff(propsArray) {
  // Edge case: features may be duplicated. See issue #122.
  // If the changeType is `added` ignore the second feature.
  if (propsArray.length === 1 || propsArray[0].changeType === 'added') {
    var changeType = propsArray[0].changeType;

    if (changeType === 'added') {
      return getAdded(propsArray[0]);
    } else {
      throw new Error('only 1 element but neither added nor deleted');
    }
  } else {
    var oldProps = getOld(propsArray);
    var newProps = getNew(propsArray);
    return getDiff(oldProps, newProps);
  }
}

function getDiff(oldProps, newProps) {
  var ret = {};

  for (var prop in newProps) {
    ret[prop] = {};

    if (!oldProps.hasOwnProperty(prop)) {
      ret[prop]['added'] = newProps[prop];
    } else {
      var oldValue = oldProps[prop];
      var newValue = newProps[prop];

      if (oldValue === newValue) {
        ret[prop]['unchanged'] = newValue;
      } else {
        ret[prop]['modifiedOld'] = oldValue;
        ret[prop]['modifiedNew'] = newValue;
      }
    }
  }

  for (var oldProp in oldProps) {
    if (!ret.hasOwnProperty(oldProp)) {
      ret[oldProp] = {
        deleted: oldProps[oldProp]
      };
    }
  }

  return ret;
}

function getAdded(props) {
  var ret = {};

  for (var prop in props) {
    ret[prop] = {
      added: props[prop]
    };
  }

  return ret;
}

function getOld(propsArray) {
  for (var i = 0; i < propsArray.length; i++) {
    var changeType = propsArray[i].changeType;

    if (changeType === 'modifiedOld' || changeType === 'deletedOld') {
      return propsArray[i];
    }
  }
}

function getNew(propsArray) {
  for (var i = 0; i < propsArray.length; i++) {
    var changeType = propsArray[i].changeType;

    if (changeType === 'modifiedNew' || changeType === 'deletedNew') {
      return propsArray[i];
    }
  }
}

var domain; // This constructor is used to store event handlers. Instantiating this is
// faster than explicitly calling `Object.create(null)` to get a "clean" empty
// object (tested with v8 v4.9).

function EventHandlers() {}

EventHandlers.prototype = Object.create(null);

function EventEmitter() {
  EventEmitter.init.call(this);
}
// require('events') === require('events').EventEmitter

EventEmitter.EventEmitter = EventEmitter;
EventEmitter.usingDomains = false;
EventEmitter.prototype.domain = undefined;
EventEmitter.prototype._events = undefined;
EventEmitter.prototype._maxListeners = undefined; // By default EventEmitters will print a warning if more than 10 listeners are
// added to it. This is a useful default which helps finding memory leaks.

EventEmitter.defaultMaxListeners = 10;

EventEmitter.init = function () {
  this.domain = null;

  if (EventEmitter.usingDomains) {
    // if there is an active domain, then attach to it.
    if (domain.active ) ;
  }

  if (!this._events || this._events === Object.getPrototypeOf(this)._events) {
    this._events = new EventHandlers();
    this._eventsCount = 0;
  }

  this._maxListeners = this._maxListeners || undefined;
}; // Obviously not all Emitters should be limited to 10. This function allows
// that to be increased. Set to zero for unlimited.


EventEmitter.prototype.setMaxListeners = function setMaxListeners(n) {
  if (typeof n !== 'number' || n < 0 || isNaN(n)) throw new TypeError('"n" argument must be a positive number');
  this._maxListeners = n;
  return this;
};

function $getMaxListeners(that) {
  if (that._maxListeners === undefined) return EventEmitter.defaultMaxListeners;
  return that._maxListeners;
}

EventEmitter.prototype.getMaxListeners = function getMaxListeners() {
  return $getMaxListeners(this);
}; // These standalone emit* functions are used to optimize calling of event
// handlers for fast cases because emit() itself often has a variable number of
// arguments and can be deoptimized because of that. These functions always have
// the same number of arguments and thus do not get deoptimized, so the code
// inside them can execute faster.


function emitNone(handler, isFn, self) {
  if (isFn) handler.call(self);else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);

    for (var i = 0; i < len; ++i) listeners[i].call(self);
  }
}

function emitOne(handler, isFn, self, arg1) {
  if (isFn) handler.call(self, arg1);else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);

    for (var i = 0; i < len; ++i) listeners[i].call(self, arg1);
  }
}

function emitTwo(handler, isFn, self, arg1, arg2) {
  if (isFn) handler.call(self, arg1, arg2);else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);

    for (var i = 0; i < len; ++i) listeners[i].call(self, arg1, arg2);
  }
}

function emitThree(handler, isFn, self, arg1, arg2, arg3) {
  if (isFn) handler.call(self, arg1, arg2, arg3);else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);

    for (var i = 0; i < len; ++i) listeners[i].call(self, arg1, arg2, arg3);
  }
}

function emitMany(handler, isFn, self, args) {
  if (isFn) handler.apply(self, args);else {
    var len = handler.length;
    var listeners = arrayClone(handler, len);

    for (var i = 0; i < len; ++i) listeners[i].apply(self, args);
  }
}

EventEmitter.prototype.emit = function emit(type) {
  var er, handler, len, args, i, events, domain;
  var doError = type === 'error';
  events = this._events;
  if (events) doError = doError && events.error == null;else if (!doError) return false;
  domain = this.domain; // If there is no 'error' event listener then throw.

  if (doError) {
    er = arguments[1];

    if (domain) {
      if (!er) er = new Error('Uncaught, unspecified "error" event');
      er.domainEmitter = this;
      er.domain = domain;
      er.domainThrown = false;
      domain.emit('error', er);
    } else if (er instanceof Error) {
      throw er; // Unhandled 'error' event
    } else {
      // At least give some kind of context to the user
      var err = new Error('Uncaught, unspecified "error" event. (' + er + ')');
      err.context = er;
      throw err;
    }

    return false;
  }

  handler = events[type];
  if (!handler) return false;
  var isFn = typeof handler === 'function';
  len = arguments.length;

  switch (len) {
    // fast cases
    case 1:
      emitNone(handler, isFn, this);
      break;

    case 2:
      emitOne(handler, isFn, this, arguments[1]);
      break;

    case 3:
      emitTwo(handler, isFn, this, arguments[1], arguments[2]);
      break;

    case 4:
      emitThree(handler, isFn, this, arguments[1], arguments[2], arguments[3]);
      break;
    // slower

    default:
      args = new Array(len - 1);

      for (i = 1; i < len; i++) args[i - 1] = arguments[i];

      emitMany(handler, isFn, this, args);
  }
  return true;
};

function _addListener(target, type, listener, prepend) {
  var m;
  var events;
  var existing;
  if (typeof listener !== 'function') throw new TypeError('"listener" argument must be a function');
  events = target._events;

  if (!events) {
    events = target._events = new EventHandlers();
    target._eventsCount = 0;
  } else {
    // To avoid recursion in the case that type === "newListener"! Before
    // adding it to the listeners, first emit "newListener".
    if (events.newListener) {
      target.emit('newListener', type, listener.listener ? listener.listener : listener); // Re-assign `events` because a newListener handler could have caused the
      // this._events to be assigned to a new object

      events = target._events;
    }

    existing = events[type];
  }

  if (!existing) {
    // Optimize the case of one listener. Don't need the extra array object.
    existing = events[type] = listener;
    ++target._eventsCount;
  } else {
    if (typeof existing === 'function') {
      // Adding the second element, need to change to array.
      existing = events[type] = prepend ? [listener, existing] : [existing, listener];
    } else {
      // If we've already got an array, just append.
      if (prepend) {
        existing.unshift(listener);
      } else {
        existing.push(listener);
      }
    } // Check for listener leak


    if (!existing.warned) {
      m = $getMaxListeners(target);

      if (m && m > 0 && existing.length > m) {
        existing.warned = true;
        var w = new Error('Possible EventEmitter memory leak detected. ' + existing.length + ' ' + type + ' listeners added. ' + 'Use emitter.setMaxListeners() to increase limit');
        w.name = 'MaxListenersExceededWarning';
        w.emitter = target;
        w.type = type;
        w.count = existing.length;
        emitWarning(w);
      }
    }
  }

  return target;
}

function emitWarning(e) {
  typeof console.warn === 'function' ? console.warn(e) : console.log(e);
}

EventEmitter.prototype.addListener = function addListener(type, listener) {
  return _addListener(this, type, listener, false);
};

EventEmitter.prototype.on = EventEmitter.prototype.addListener;

EventEmitter.prototype.prependListener = function prependListener(type, listener) {
  return _addListener(this, type, listener, true);
};

function _onceWrap(target, type, listener) {
  var fired = false;

  function g() {
    target.removeListener(type, g);

    if (!fired) {
      fired = true;
      listener.apply(target, arguments);
    }
  }

  g.listener = listener;
  return g;
}

EventEmitter.prototype.once = function once(type, listener) {
  if (typeof listener !== 'function') throw new TypeError('"listener" argument must be a function');
  this.on(type, _onceWrap(this, type, listener));
  return this;
};

EventEmitter.prototype.prependOnceListener = function prependOnceListener(type, listener) {
  if (typeof listener !== 'function') throw new TypeError('"listener" argument must be a function');
  this.prependListener(type, _onceWrap(this, type, listener));
  return this;
}; // emits a 'removeListener' event iff the listener was removed


EventEmitter.prototype.removeListener = function removeListener(type, listener) {
  var list, events, position, i, originalListener;
  if (typeof listener !== 'function') throw new TypeError('"listener" argument must be a function');
  events = this._events;
  if (!events) return this;
  list = events[type];
  if (!list) return this;

  if (list === listener || list.listener && list.listener === listener) {
    if (--this._eventsCount === 0) this._events = new EventHandlers();else {
      delete events[type];
      if (events.removeListener) this.emit('removeListener', type, list.listener || listener);
    }
  } else if (typeof list !== 'function') {
    position = -1;

    for (i = list.length; i-- > 0;) {
      if (list[i] === listener || list[i].listener && list[i].listener === listener) {
        originalListener = list[i].listener;
        position = i;
        break;
      }
    }

    if (position < 0) return this;

    if (list.length === 1) {
      list[0] = undefined;

      if (--this._eventsCount === 0) {
        this._events = new EventHandlers();
        return this;
      } else {
        delete events[type];
      }
    } else {
      spliceOne(list, position);
    }

    if (events.removeListener) this.emit('removeListener', type, originalListener || listener);
  }

  return this;
};

EventEmitter.prototype.removeAllListeners = function removeAllListeners(type) {
  var listeners, events;
  events = this._events;
  if (!events) return this; // not listening for removeListener, no need to emit

  if (!events.removeListener) {
    if (arguments.length === 0) {
      this._events = new EventHandlers();
      this._eventsCount = 0;
    } else if (events[type]) {
      if (--this._eventsCount === 0) this._events = new EventHandlers();else delete events[type];
    }

    return this;
  } // emit removeListener for all listeners on all events


  if (arguments.length === 0) {
    var keys = Object.keys(events);

    for (var i = 0, key; i < keys.length; ++i) {
      key = keys[i];
      if (key === 'removeListener') continue;
      this.removeAllListeners(key);
    }

    this.removeAllListeners('removeListener');
    this._events = new EventHandlers();
    this._eventsCount = 0;
    return this;
  }

  listeners = events[type];

  if (typeof listeners === 'function') {
    this.removeListener(type, listeners);
  } else if (listeners) {
    // LIFO order
    do {
      this.removeListener(type, listeners[listeners.length - 1]);
    } while (listeners[0]);
  }

  return this;
};

EventEmitter.prototype.listeners = function listeners(type) {
  var evlistener;
  var ret;
  var events = this._events;
  if (!events) ret = [];else {
    evlistener = events[type];
    if (!evlistener) ret = [];else if (typeof evlistener === 'function') ret = [evlistener.listener || evlistener];else ret = unwrapListeners(evlistener);
  }
  return ret;
};

EventEmitter.listenerCount = function (emitter, type) {
  if (typeof emitter.listenerCount === 'function') {
    return emitter.listenerCount(type);
  } else {
    return listenerCount.call(emitter, type);
  }
};

EventEmitter.prototype.listenerCount = listenerCount;

function listenerCount(type) {
  var events = this._events;

  if (events) {
    var evlistener = events[type];

    if (typeof evlistener === 'function') {
      return 1;
    } else if (evlistener) {
      return evlistener.length;
    }
  }

  return 0;
}

EventEmitter.prototype.eventNames = function eventNames() {
  return this._eventsCount > 0 ? Reflect.ownKeys(this._events) : [];
}; // About 1.5x faster than the two-arg version of Array#splice().


function spliceOne(list, index) {
  for (var i = index, k = i + 1, n = list.length; k < n; i += 1, k += 1) list[i] = list[k];

  list.pop();
}

function arrayClone(arr, i) {
  var copy = new Array(i);

  while (i--) copy[i] = arr[i];

  return copy;
}

function unwrapListeners(arr) {
  var ret = new Array(arr.length);

  for (var i = 0; i < ret.length; ++i) {
    ret[i] = arr[i].listener || arr[i];
  }

  return ret;
}

function getBounds(bbox) {
  var left = +bbox.left,
      right = +bbox.right,
      top = +bbox.top,
      bottom = +bbox.bottom;
  return new mapboxgl.LngLatBounds(new mapboxgl.LngLat(left, bottom), new mapboxgl.LngLat(right, top));
}
function getBoundingBox(bounds) {
  var left = bounds.getWest(),
      right = bounds.getEast(),
      top = bounds.getNorth(),
      bottom = bounds.getSouth();
  var padX = 0;
  var padY = 0;

  if (!(left === -180 && right === 180 && top === 90 && bottom === -90)) {
    padX = Math.max((right - left) / 5, 0.0001);
    padY = Math.max((top - bottom) / 5, 0.0001);
  }

  var bboxPolygon = turfBboxPolygon([left - padX, bottom - padY, right + padX, top + padY]);
  return featureCollection([bboxPolygon]);
}
function getFeatureBBOX(features) {
  // creates a GeoJSON with the features and calculate its BBOX
  return bbox({
    type: 'FeatureCollection',
    features: features
  });
}

function getFeatures(features) {
  var keys = Object.keys(features);
  return keys.map(item => features[item]);
}
function processFeatures(features) {
  const finalReport = new Map();
  const analyzedFeatures = features.map(feature => analyzeFeature(feature[0], feature[1]));
  const keys = ['addedTags', 'changedValues', 'deletedTags'];
  analyzedFeatures.map(item => keys.map(key => item.get(key).forEach(tag => {
    if (finalReport.get(tag)) {
      finalReport.set(tag, finalReport.get(tag).concat([item.get('id')]));
    } else {
      finalReport.set(tag, [item.get('id')]);
    }
  })));
  return finalReport;
}
function analyzeFeature(newVersion, oldVersion) {
  var oldVersionKeys = Object.keys(oldVersion.properties.tags);
  var newVersionKeys = Object.keys(newVersion.properties.tags);
  var addedTags = newVersionKeys.filter(tag => oldVersionKeys.indexOf(tag) === -1);
  var deletedTags = oldVersionKeys.filter(tag => newVersionKeys.indexOf(tag) === -1);
  var changedValues = newVersionKeys.filter(tag => addedTags.indexOf(tag) === -1 && deletedTags.indexOf(tag) === -1).filter(tag => newVersion.properties.tags[tag] !== oldVersion.properties.tags[tag]);
  var result = new Map();
  result.set('id', newVersion.properties.id).set('addedTags', addedTags.map(tag => `Added tag ${tag}`)).set('deletedTags', deletedTags.map(tag => `Deleted tag ${tag}`)).set('changedValues', changedValues.map(tag => `Changed value of tag ${tag}`));
  return result;
}
function selectFeature(id) {
  if (!id || !cmap) return;
  cmap.emit('selectFeature', 'node|way', id);
}

function FeatureListItem(props) {
  return /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("span", {
    className: "cmap-pointer ",
    onClick: () => selectFeature(props.id)
  }, props.id));
}

class ChangeItem extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      opened: false
    };
    this.tag = props.change[0];
    this.value = props.change[1];
    this.handleChange = this.handleChange.bind(this);
  }

  handleChange() {
    this.setState({
      opened: !this.state.opened
    });
  }

  render() {
    return /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("h7", {
      className: "cmap-sub-heading cmap-pointer",
      onClick: this.handleChange
    }, this.state.opened ? '▼' : '▶', this.tag), /*#__PURE__*/React.createElement("ul", {
      className: "cmap-vlist",
      style: {
        display: this.state.opened ? 'block' : 'none'
      }
    }, this.value.map((id, k) => /*#__PURE__*/React.createElement(FeatureListItem, {
      id: id,
      key: k
    }))));
  }

}
class Sidebar extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      actions: true,
      type: false,
      changes: false,
      mapStyle: false,
      user: false
    };
    this.changeReport = [];
    this.changedFeatures = processFeatures(getFeatures(this.props.result.featureMap).filter(item => item.length === 2 && item[0].properties.action === 'modify')).forEach((featureIDs, tag) => this.changeReport.push([tag, featureIDs]));
    this.toggleUser = this.toggleUser.bind(this);
    this.toggleActions = this.toggleActions.bind(this);
    this.toggleType = this.toggleType.bind(this);
    this.toggleChanges = this.toggleChanges.bind(this);
    this.toggleMapStyle = this.toggleMapStyle.bind(this);
  }

  toggleUser() {
    this.setState({
      user: !this.state.user
    });
  }

  toggleActions() {
    this.setState({
      actions: !this.state.actions
    });
  }

  toggleType() {
    this.setState({
      type: !this.state.type
    });
  }

  toggleChanges() {
    this.setState({
      changes: !this.state.changes
    });
  }

  toggleMapStyle() {
    this.setState({
      mapStyle: !this.state.mapStyle
    });
  }

  render() {
    const result = this.props.result;
    const changesetId = this.props.changesetId;
    const filterLayers = this.props.filterLayers;
    var date = new Date(result.changeset.to ? result.changeset.to : result.changeset.from);
    var bbox = result.changeset.bbox;
    var bounds = getBounds(bbox);
    var center = bounds.getCenter();
    var userName = result.changeset.user;
    var userId = result.changeset.uid;
    return /*#__PURE__*/React.createElement("div", {
      className: "cmap-sidebar"
    }, /*#__PURE__*/React.createElement("section", {
      className: "cmap-changeset-section cmap-fill-light cmap-pt3"
    }, /*#__PURE__*/React.createElement("h6", {
      className: "cmap-heading"
    }, "Changeset:", /*#__PURE__*/React.createElement("em", {
      className: "cmap-changeset-id"
    }, changesetId), /*#__PURE__*/React.createElement("small", {
      className: "cmap-time",
      title: date
    }, "(", moment(date).fromNow(), ")")), /*#__PURE__*/React.createElement("ul", {
      className: "cmap-hlist"
    }, /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-c-link-osm",
      href: 'https://openstreetmap.org/changeset/' + changesetId
    }, "OSM")), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-c-link-osmcha",
      href: 'https://osmcha.org/changesets/' + changesetId + '/'
    }, "OSMCha")), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-c-link-achavi",
      href: 'https://overpass-api.de/achavi/?changeset=' + changesetId
    }, "Achavi")), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-c-link-osmhv",
      href: 'http://osmhv.openstreetmap.de/changeset.jsp?id=' + changesetId
    }, "OSM HV")), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-c-link-josm",
      href: 'http://127.0.0.1:8111/import?url=http://www.openstreetmap.org/api/0.6/changeset/' + changesetId + '/download'
    }, "JOSM")), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-c-link-id",
      href: 'http://preview.ideditor.com/release#map=15/' + center.lat + '/' + center.lng
    }, "iD")))), /*#__PURE__*/React.createElement("section", {
      className: "cmap-user-section cmap-fill-light cmap-pb3"
    }, /*#__PURE__*/React.createElement("h6", {
      className: "cmap-heading",
      onClick: this.toggleUser
    }, this.state.user ? '▼' : '▶', "User: ", /*#__PURE__*/React.createElement("em", {
      className: "cmap-user-id"
    }, userName)), /*#__PURE__*/React.createElement("ul", {
      className: "cmap-hlist",
      style: {
        display: this.state.user ? 'block' : 'none'
      }
    }, /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-u-link-osm",
      href: 'https://openstreetmap.org/user/' + userName
    }, "OSM")), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-u-link-hdyc",
      href: 'http://hdyc.neis-one.org/?' + userName
    }, "HDYC")), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-u-link-disc",
      href: 'http://resultmaps.neis-one.org/osm-discussion-comments?uid=' + userId
    }, "Discussions")), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      className: "cmap-hlist-item cmap-noselect cmap-pointer cmap-u-link-comm",
      href: 'http://resultmaps.neis-one.org/osm-discussion-comments?uid=115894' + userId + '&commented'
    }, "Comments")))), /*#__PURE__*/React.createElement("section", {
      className: "cmap-filter-action-section cmap-pt3"
    }, /*#__PURE__*/React.createElement("h6", {
      className: "cmap-heading pointer",
      onClick: this.toggleActions
    }, this.state.actions ? '▼' : '▶', "Filter by actions"), /*#__PURE__*/React.createElement("ul", {
      style: {
        display: this.state.actions ? 'block' : 'none'
      },
      className: "cmap-hlist"
    }, /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "checkbox",
      value: "added",
      defaultChecked: "true",
      id: "cmap-layer-selector-added",
      onChange: filterLayers
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Added"), /*#__PURE__*/React.createElement("span", {
      className: "cmap-color-box cmap-color-added"
    }))), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "checkbox",
      value: "modified",
      defaultChecked: "true",
      id: "cmap-layer-selector-modified",
      onChange: filterLayers
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Modified"), /*#__PURE__*/React.createElement("span", {
      className: "cmap-color-box cmap-color-modified-old"
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-unicode"
    }, "\u2192"), /*#__PURE__*/React.createElement("span", {
      className: "cmap-color-box cmap-color-modified-new"
    }))), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "checkbox",
      value: "deleted",
      defaultChecked: "true",
      id: "cmap-layer-selector-deleted",
      onChange: filterLayers
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Deleted"), /*#__PURE__*/React.createElement("span", {
      className: "cmap-color-box cmap-color-deleted"
    }))))), /*#__PURE__*/React.createElement("section", {
      className: "cmap-filter-type-section"
    }, /*#__PURE__*/React.createElement("h6", {
      className: "cmap-heading pointer",
      onClick: this.toggleType
    }, this.state.type ? '▼' : '▶', "Filter by type"), /*#__PURE__*/React.createElement("ul", {
      className: "cmap-hlist",
      style: {
        display: this.state.type ? 'block' : 'none'
      }
    }, /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "checkbox",
      value: "nodes",
      defaultChecked: "true",
      id: "cmap-type-selector-nodes",
      onClick: filterLayers
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Nodes"))), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "checkbox",
      value: "ways",
      defaultChecked: "true",
      id: "cmap-type-selector-ways",
      onChange: filterLayers
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Ways"))), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "checkbox",
      value: "relations",
      defaultChecked: "true",
      id: "cmap-type-selector-relations",
      onChange: filterLayers
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Relations"))))), /*#__PURE__*/React.createElement("section", {
      className: "cmap-filter-changes-section cmap-pb3"
    }, /*#__PURE__*/React.createElement("h6", {
      className: "cmap-heading pointer",
      onClick: this.toggleChanges
    }, this.state.changes ? '▼' : '▶', "Tags added / updated / deleted"), /*#__PURE__*/React.createElement("ul", {
      className: "cmap-sub-hlist",
      style: {
        display: this.state.changes ? 'block' : 'none'
      }
    }, this.changeReport.sort().map((change, k) => /*#__PURE__*/React.createElement(ChangeItem, {
      key: k,
      change: change
    })))), /*#__PURE__*/React.createElement("section", {
      className: "cmap-map-style-section cmap-pb3"
    }, /*#__PURE__*/React.createElement("h6", {
      className: "cmap-heading pointer",
      onClick: this.toggleMapStyle
    }, this.state.mapStyle ? '▼' : '▶', "Map style"), /*#__PURE__*/React.createElement("ul", {
      className: "cmap-hlist",
      style: {
        display: this.state.mapStyle ? 'block' : 'none'
      }
    }, /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "radio",
      value: "satellite",
      defaultChecked: "true",
      name: "baselayer",
      id: "cmap-baselayer-satellite",
      onChange: this.props.toggleLayer
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Satellite"))), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "radio",
      value: "streets",
      name: "baselayer",
      id: "cmap-baselayer-streets",
      onChange: this.props.toggleLayer
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Streets"))), /*#__PURE__*/React.createElement("li", null, /*#__PURE__*/React.createElement("label", {
      className: "cmap-hlist-item cmap-noselect cmap-pointer"
    }, /*#__PURE__*/React.createElement("input", {
      type: "radio",
      value: "dark",
      name: "baselayer",
      id: "cmap-baselayer-dark",
      onChange: this.props.toggleLayer
    }), /*#__PURE__*/React.createElement("span", {
      className: "cmap-label-text"
    }, "Dark"))))));
  }

}

// The app has a window to render DOM content in.
var gui = typeof navigator !== 'undefined' && typeof window !== 'undefined'; // NW.JS' background script loads into invisible empty HTML page.
// Therefore NW.JS "main" script has all of window, document, navigator objects
// and window.open() won't be null if App's window is launched from such "main".

if (gui && typeof nw !== 'undefined') {
  try {
    nw.Window.get();
  } catch (err) {
    gui = false;
  }
} // The app runs in terminal/console and can only use console.log.


var terminal = !gui;
var ua = gui ? navigator.userAgent : undefined;
function registerQuery(query, handler) {
  var mql = window.matchMedia(query);
  handler(mql.matches);

  var listener = () => handler(mql.matches);

  mql.addListener(listener);
  return () => mql.removeListener(listener);
} // The holy grail

var p = {
  gui,
  terminal,
  registerQuery
};

// it is true for Node.js, Electron, NW.JS
// it is false for browsers with shims or bundles of some Node modules (shimmed process, EventEmitter, etc..)

p.node = typeof process !== 'undefined' && !!process.versions && !!process.versions.node; // Detects if the APP is launched as standalone Progressive Web App. Still a website, but looking like OS app, without url bar.
// TODO: WARNING: this returns true even for popup windows created with window.open()

p.pwa = p.gui && window.matchMedia('(display-mode: standalone)').matches && document.head.querySelector('[rel="manifest"]') !== null; // Windows 10 app - Universal Windows Platform.

p.uwp = typeof Windows !== 'undefined' && typeof MSApp !== 'undefined'; // Node + Chromium

p.nwjs = !!(p.node && process.versions.nw);
p.electron = !!(p.node && process.versions.electron); // Cordova mobile app

p.cordova = !!(p.gui && window.cordova); // The platform requires app to be compiled, bundled or packaged.

p.packaged = p.uwp || p.nwjs || p.electron || p.cordova; // The app runs inside browser and is served from a server or browser cache.

p.web = !p.node && !p.packaged;
p.browser = p.web; // alias
// App is a plain old webpage and not a PWA.

p.website = p.web && !p.pwa; // Script is executed inside Web Worker

p.worker = !p.gui && typeof self !== 'undefined' && self.importScripts !== undefined; //&& self.close !== undefined

p.serviceWorker = p.worker && !!navigator.serviceWorker.controller || false;

// TODO: figure out if these should be mutually exclusive (Tizen and Android are based on linux and have Linux in their user agents)

p.android = p.gui ? ua.includes('Android') : false;
p.chromeos = p.gui ? ua.includes('CrOS') : false;
p.tizen = p.gui ? ua.includes('Tizen') : false;
p.ios = p.gui && /iPad|iPhone|iPod/.test(ua) && !window.MSStream || false;
p.linuxBased = p.android || p.tizen;
p.windows = p.node ? process.platform === 'win32' : ua.includes('Windows');
p.macos = p.node ? process.platform === 'darwin' : ua.includes('Macintosh');
p.linux = p.node ? process.platform === 'linux' : ua.includes('Linux') && !p.linuxBased && !p.macos;

// I.e: UWP apps are rendered by Edge's EdgeHTML, Electron/NWJS apps are rendered by Chromium.
// https://blogs.windows.com/msedgedev/2017/10/05/microsoft-edge-ios-android-developer/#49Fi4TfpgzHAwuXQ.97
// https://www.whatismybrowser.com/guides/the-latest-user-agent/edge
// https://www.whatismybrowser.com/guides/the-latest-user-agent/chrome
// https://www.whatismybrowser.com/guides/the-latest-user-agent/safari
// https://www.whatismybrowser.com/guides/the-latest-user-agent/opera
// https://www.whatismybrowser.com/guides/the-latest-user-agent/firefox

p.edgeHtml = p.gui && ua.includes('Edge/');
p.edgeChromium = p.gui && ua.includes('Edg/');
p.edgeAndroid = p.gui && ua.includes('EdgA/');
p.edgeIos = p.gui && ua.includes('EdgiOS/');
p.chromeIos = p.gui && ua.includes('CriOS/');
p.firefoxIos = p.gui && ua.includes('FxiOS/');
p.edge = p.edgeHtml || p.edgeChromium || p.edgeAndroid || p.edgeIos;
p.samsungBrowser = p.gui && ua.includes('SamsungBrowser/');
p.opera = p.gui && (ua.includes('Opera') || ua.includes('OPR/'));
p.firefox = p.gui && (ua.includes('Firefox') || p.firefoxIos);
p.chrome = p.gui && (ua.includes('Chrome') || p.chromeIos) && !p.edge && !p.opera && !p.samsungBrowser;
p.safari = p.gui && ua.includes('Safari') && !p.chrome && !p.edge && !p.firefox && !p.opera || p.edgeIos || p.chromeIos || p.firefoxIos;
p.ie = p.trident = p.gui && ua.includes('Trident');
p.blink = p.chrome && !p.chromeIos || p.edgeChromium || p.edgeAndroid || p.samsungBrowser;
p.webkit = p.blink || p.safari;
p.gecko = p.firefox && !p.firefoxIos && !p.webkit && !p.safari; // even android firefox now uses gecko

var listeners = {};

p.on = function (name, listener) {
  listeners[name] = listeners[name] || new Set();
  listeners[name].add(listener);
};

p.off = p.removeListener = function (name, listener) {
  if (listeners[name]) listeners[name].delete(listener);
};

var emit = p.emit = function (name, value) {
  if (listeners[name]) listeners[name].forEach(listener => listener(value));
};

function applyAndEmit(key, newVal) {
  if (p[key] === newVal) return false;
  p[key] = newVal;
  emit(key, newVal);
  return true;
}

if (p.gui) {
  p.pixelRatio = parseFloat(window.devicePixelRatio.toFixed(2));
  p.gameconsole = ua.includes('Xbox') || ua.includes('PlayStation');
  var gamepadCount = 0;
  window.addEventListener('gamepadconnected', e => gamepadCount++);
  window.addEventListener('gamepaddisconnected', e => gamepadCount--);

  if (p.gameconsole) {
    // WARNING: do not use Gamepad API unless necessary. On Xbox it automatically hides pointer and shows warning that the site uses
    //          custom gamepad controls. We don't want to trigger that. This library only observes.
    p.gamepad = true;
    p.mouse = true; // this is controvertial. if app decides to use gamepad controls, then it doesnt have the mouse & hover anymore

    p.touch = false;
    p.tv = true;
    p.battery = false;
    p.phone = p.tablet = p.hybrid = p.laptop = p.desktop = false;
    p.formfactor = 'gameconsole';
  } else {
    p.touch = navigator.maxTouchPoints > 0;
    p.tv = ua.includes('TV'); // TODO: detect when gamepad is custom handled (and used not just used to control the emulated pointer on smart TVs)
    // TODO: throw gamepad into the platform.input
    // warning: older browsers filled used undefined instead of null

    gamepadCount = navigator.getGamepads ? Array.from(navigator.getGamepads()).filter(g => g !== null && g !== undefined).length : 0;
    p.gamepad = gamepadCount > 0;
    registerQuery('(orientation: portrait)', bool => {
      p.portrait = bool;
      p.landscape = !bool;
      p.orientation = bool ? 'portrait' : 'landscape';
      emit('portrait', p.portrait);
      emit('landscape', p.landscape);
      emit('orientation', p.orientation);
    });
    registerQuery('(any-pointer: coarse)', bool => {
      applyAndEmit('touch', bool);
      let formFactorChanged = applyAndEmit('formfactor', getFormfactor());
      if (formFactorChanged) applyFormFactor();
    });
    registerQuery('(hover: hover)', bool => {
      applyAndEmit('mouse', bool);
      applyAndEmit('input', bool ? 'mouse' : 'touch');
      let formFactorChanged = applyAndEmit('formfactor', getFormfactor());
      if (formFactorChanged) applyFormFactor();
    });

    function applyFormFactor() {
      applyAndEmit('tv', p.formfactor === 'tv');
      applyAndEmit('phone', p.formfactor === 'phone');
      applyAndEmit('tablet', p.formfactor === 'tablet');
      applyAndEmit('hybrid', p.formfactor === 'hybrid');
      applyAndEmit('laptop', p.formfactor === 'laptop');
      applyAndEmit('desktop', p.formfactor === 'desktop');
    }

    function getFormfactor() {
      var shorterScreenSide = Math.min(window.screen.width, window.screen.height); // TODO: add 'iot' or some other form of window-less app or monitor-less hardware.

      if (p.tv) return 'tv';else if (p.touch && shorterScreenSide < 600) return 'phone';else if (p.touch && !p.mouse) return 'tablet';else if (p.touch && p.mouse) return 'hybrid';else if (p.battery) return 'laptop';else return 'desktop';
    }
  }
}

p.csp = p.uwp || false; // Detects if NW.JS runs in SDK version (console available) and if Electron is executed from npm/node_modules/electron global.

if (p.nwjs) {
  p.dev = process.versions['nw-flavor'] === 'sdk';
} else if (p.electron) {
  p.dev = process.execPath.replace(/\\/g, '/').includes('node_modules/electron/');
} else if (p.uwp) {
  p.dev = Windows.ApplicationModel.Package.current.isDevelopmentMode;
} else if (p.node) {
  p.dev = "production" !== 'production'; //} else if (p.web) {
  //	p.dev = undefined // no longer working in latest chrome
} else {
  // TODO: would be nice to detect if UWP is attached to Visual Studio debugger.
  p.dev = undefined;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

class DropdownContent extends React.PureComponent {
  constructor(props) {
    super(props);

    this.isActive = obj => {
      for (let v of this.props.value) {
        if (v.label === obj.label) {
          return true;
        }
      }

      return false;
    };

    this.handleClick = data => {
      if (data) {
        var label = data.label;
        if (!label || !this.props.value || !this.props.onChange) return;
        const value = this.props.value;
        let ourObj = data;
        if (!ourObj) return;
        let isRemove = false;

        for (let x = 0; x < value.length; x++) {
          if (value[x].label === label) {
            isRemove = true;
            this.props.onRemove(ourObj);
            this.props.onChange(value.slice(0, x).concat(value.slice(x + 1)));
          }
        }

        if (!isRemove) {
          let newArray = value.slice(0, value.length);

          if (!this.props.multi) {
            newArray = [];
          }

          newArray.push(ourObj);
          this.props.onAdd(ourObj);
          this.props.onChange(newArray);
        }
      }

      if (!this.props.multi) {
        this.props.toggleDropdown();
      }
    };
  }

  render() {
    return /*#__PURE__*/React.createElement("div", {
      className: "cmap-dropdown-content"
    }, this.props.options.map((i, k) => /*#__PURE__*/React.createElement("span", {
      key: k,
      onClick: this.handleClick.bind(null, i)
    }, this.props.multi && /*#__PURE__*/React.createElement("input", {
      "data-label": i.label,
      "data-payload": JSON.stringify(i),
      type: "checkbox",
      checked: this.isActive(i),
      value: i.label
    }), i.href ? /*#__PURE__*/React.createElement("a", {
      target: '_blank',
      href: i.href,
      onClick: this.props.toggleDropdown,
      className: `${this.isActive(i) ? 'is-active' : ''}`
    }, i.label) : /*#__PURE__*/React.createElement("a", {
      onClick: this.props.toggleDropdown,
      className: `${this.isActive(i) ? 'is-active' : ''}`
    }, i.label), this.props.deletable && /*#__PURE__*/React.createElement("a", {
      onClick: e => {
        e.preventDefault();
        e.stopPropagation();
        this.props.toggleDropdown();
        this.props.deletable(i.value);
      }
    }, "x"))));
  }

}

DropdownContent.propTypes = {
  className: PropTypes.string,
  disabled: PropTypes.bool,
  value: PropTypes.array,
  onChange: PropTypes.function,
  onAdd: PropTypes.function,
  onRemove: PropTypes.function,
  toggleDropdown: PropTypes.function,
  options: PropTypes.array,
  display: PropTypes.string,
  deletable: PropTypes.function,
  multi: PropTypes.bool
};
DropdownContent.defaultProps = {
  value: [],
  onAdd: () => {},
  onRemove: () => {}
};
class _Dropdown extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      display: false
    };

    this.handleClickOutside = () => {
      this.setState({
        display: false
      });
    };

    this.toggleDropdown = () => {
      this.setState({
        display: !this.state.display
      });
    };

    this.isActive = obj => {
      for (let v of this.props.value) {
        if (v.label === obj.label) {
          return true;
        }
      }

      return false;
    };
  }

  render() {
    return /*#__PURE__*/React.createElement("div", {
      className: `cmap-dropdown ${this.props.className || ''}`
    }, /*#__PURE__*/React.createElement("button", {
      className: "cmap-dropbtn",
      onClick: this.toggleDropdown
    }, /*#__PURE__*/React.createElement("span", null, this.props.display), /*#__PURE__*/React.createElement("span", {
      className: "cmap-dropdown-icon"
    })), this.state.display && /*#__PURE__*/React.createElement(DropdownContent, _extends({}, this.props, {
      toggleDropdown: this.toggleDropdown
    })));
  }

}
_Dropdown.propTypes = {
  className: PropTypes.string,
  value: PropTypes.array,
  display: PropTypes.string
};
const Dropdown = onClickOutside(_Dropdown);

const displayDiff = function (id, featureMap, metadataContainer, tagsContainer, membersContainer) {
  var featuresWithId = featureMap[id];
  render$1( /*#__PURE__*/React.createElement(MetadataTable, {
    featuresWithId: featuresWithId,
    id: id
  }), metadataContainer);
  render$1( /*#__PURE__*/React.createElement(TagsTable, {
    featuresWithId: featuresWithId
  }), tagsContainer);

  if (featuresWithId[0].properties.type === 'relation') {
    document.querySelector('.cmap-diff-members').style.display = 'block';
    render$1( /*#__PURE__*/React.createElement(RelationMembersTable, {
      featuresWithId: featuresWithId
    }), membersContainer);
  }
};

const MetadataTable = function ({
  featuresWithId,
  id
}) {
  const type = featuresWithId[0].properties.type;
  const metadataProps = featuresWithId.map(function (f) {
    var filteredProps = Object.assign({}, f.properties);
    delete filteredProps.tags;
    delete filteredProps.tagsCount;
    delete filteredProps.relations;
    delete filteredProps.action;
    return filteredProps;
  });
  const metadataHeader = /*#__PURE__*/React.createElement("div", {
    className: "cmap-space-between"
  }, /*#__PURE__*/React.createElement("span", null, type.toUpperCase(), ": ", id), /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(Dropdown, {
    display: "History",
    options: [{
      label: 'OSM',
      href: `https://www.openstreetmap.org/${type}/${id}/history`
    }, {
      label: 'Deep History',
      href: `https://osmlab.github.io/osm-deep-history/#/${type}/${id}`
    }, {
      label: 'PeWu',
      href: `https://pewu.github.io/osm-history/#/${type}/${id}`
    }]
  }), /*#__PURE__*/React.createElement(Dropdown, {
    display: "Open feature",
    options: [{
      label: 'iD',
      href: `https://www.openstreetmap.org/edit?editor=id&${type}=${id}`
    }, {
      label: 'JOSM',
      href: `http://127.0.0.1:8111/load_object?new_layer=true&objects=${type[0]}${id}`
    }, {
      label: 'Level0',
      href: `http://level0.osmz.ru/?url=${type}/${id}`
    }]
  })));
  return /*#__PURE__*/React.createElement(DiffTable, {
    diff: propsDiff(metadataProps),
    ignoreList: ['id', 'type', 'changeType'],
    header: metadataHeader
  });
};

MetadataTable.propTypes = {
  featuresWithId: PropTypes.array.isRequired,
  id: PropTypes.string.isRequired
};

const TagsTable = function ({
  featuresWithId
}) {
  const tagProps = featuresWithId.map(function (f) {
    const filteredProps = Object.assign({}, f.properties.tags);
    filteredProps.changeType = f.properties.changeType;
    return filteredProps;
  });
  const tagHeader = /*#__PURE__*/React.createElement("span", {
    className: "cmap-inline-block"
  }, 'Tag details'.toUpperCase());
  return /*#__PURE__*/React.createElement(DiffTable, {
    diff: propsDiff(tagProps),
    ignoreList: ['id', 'changeType'],
    header: tagHeader
  });
};

TagsTable.propTypes = {
  featuresWithId: PropTypes.array.isRequired
};

const RelationMembersTable = function ({
  featuresWithId
}) {
  return /*#__PURE__*/React.createElement("table", {
    className: "cmap-diff-table",
    style: featuresWithId[0].properties.changeType === 'added' ? {
      width: '350px'
    } : undefined
  }, /*#__PURE__*/React.createElement("thead", null, /*#__PURE__*/React.createElement("tr", null, /*#__PURE__*/React.createElement("td", {
    className: "cmap-table-head",
    colSpan: "2"
  }, /*#__PURE__*/React.createElement("span", {
    className: "cmap-strong"
  }, "MEMBERS"), " (click to highlight)"))), /*#__PURE__*/React.createElement("tbody", null, featuresWithId[0].properties.relations.map((i, n) => /*#__PURE__*/React.createElement("tr", {
    key: n,
    onClick: () => cmap.emit('selectMember', i.properties.ref)
  }, /*#__PURE__*/React.createElement("th", {
    className: "cmap-strong cmap-pointer"
  }, `${i.properties.ref} (${i.properties.type.toUpperCase()})`), /*#__PURE__*/React.createElement("td", {
    className: "diff-property cmap-scroll-styled cmap-pointer"
  }, i.properties.role)))));
};

RelationMembersTable.propTypes = {
  featuresWithId: PropTypes.array.isRequired
}; //Renders the markup for a table

const DiffTable = function ({
  diff,
  ignoreList,
  header
}) {
  var isAddedFeature = diff['changeType'].added === 'added';
  var types = ['added', 'deleted', 'modifiedOld', 'modifiedNew', 'unchanged'];
  var sortedProps = Object.keys(diff).sort(function (keyA, keyB) {
    var indexA = types.indexOf(Object.keys(diff[keyA])[0]);
    var indexB = types.indexOf(Object.keys(diff[keyB])[0]);
    return indexA - indexB;
  });
  return /*#__PURE__*/React.createElement("table", {
    className: "cmap-diff-table",
    style: isAddedFeature ? {
      width: '350px'
    } : undefined
  }, header && /*#__PURE__*/React.createElement("thead", null, /*#__PURE__*/React.createElement("tr", null, /*#__PURE__*/React.createElement("td", {
    className: "cmap-table-head",
    colSpan: isAddedFeature ? '2' : '3'
  }, header))), /*#__PURE__*/React.createElement(DiffRows, {
    diff: diff,
    sortedProps: sortedProps,
    types: types,
    isAddedFeature: isAddedFeature,
    ignoreList: ignoreList
  }));
};

DiffTable.propTypes = {
  diff: PropTypes.object.isRequired,
  ignoreList: PropTypes.array,
  header: PropTypes.node
};
DiffTable.defaultProps = {
  ignoreList: []
};

const DiffRows = function ({
  diff,
  sortedProps,
  types,
  isAddedFeature,
  ignoreList
}) {
  const rows = [];
  sortedProps.forEach(function (prop) {
    if (ignoreList.indexOf(prop) !== -1) {
      return;
    }

    const columns = [];
    types.forEach(function (type) {
      if (diff[prop].hasOwnProperty(type)) {
        const propClass = `diff-property cmap-scroll-styled props-diff-${type}`;

        if (type === 'added' && !isAddedFeature) {
          columns.push( /*#__PURE__*/React.createElement("td", {
            key: `${prop}-${type}-1`,
            className: propClass
          }));
        }

        columns.push( /*#__PURE__*/React.createElement(DiffColumn, {
          key: `${prop}-${type}-2`,
          diff: diff,
          prop: prop,
          type: type,
          propClass: propClass
        }));

        if (type === 'deleted') {
          columns.push( /*#__PURE__*/React.createElement("td", {
            key: `${prop}-${type}-3`,
            className: propClass
          }));
        } else if (type === 'unchanged') {
          columns.push( /*#__PURE__*/React.createElement(DiffColumn, {
            key: `${prop}-${type}-3`,
            diff: diff,
            prop: prop,
            type: type,
            propClass: propClass
          }));
        }
      }
    });
    rows.push( /*#__PURE__*/React.createElement("tr", {
      key: `${prop}-row`
    }, /*#__PURE__*/React.createElement("th", {
      key: `${prop}-header`,
      title: prop,
      className: "cmap-strong"
    }, prop), columns));
  });
  return /*#__PURE__*/React.createElement("tbody", null, rows);
};

DiffRows.propTypes = {
  diff: PropTypes.object.isRequired,
  sortedProps: PropTypes.array.isRequired,
  types: PropTypes.array.isRequired,
  isAddedFeature: PropTypes.bool,
  ignoreList: PropTypes.array
};
DiffRows.defaultProps = {
  isAddedFeature: false,
  ignoreList: []
};

const DiffColumn = function ({
  diff,
  prop,
  type,
  propClass
}) {
  if (prop === 'changeset' && type === 'modifiedOld') {
    return /*#__PURE__*/React.createElement("td", {
      className: propClass
    }, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      rel: "noopener noreferrer",
      className: "cmap-changeset-link",
      href: `//osmcha.org/changesets/${diff[prop][type]}`
    }, diff[prop][type]));
  } else {
    return /*#__PURE__*/React.createElement("td", {
      className: propClass
    }, diff[prop][type]);
  }
};

DiffColumn.propTypes = {
  diff: PropTypes.object.isRequired,
  prop: PropTypes.string.isRequired,
  type: PropTypes.string.isRequired,
  propClass: PropTypes.string
};

const LAYERS_KEY = {
  'added-line': {
    added: true,
    ways: true
  },
  'added-point-tagged': {
    added: true,
    nodes: true
  },
  'added-point-untagged': {
    added: true,
    nodes: true
  },
  'added-relation': {
    added: true,
    relations: true
  },
  'modified-old-line': {
    modified: true,
    ways: true
  },
  'modified-old-point-tagged': {
    modified: true,
    nodes: true
  },
  'modified-old-point-untagged': {
    modified: true,
    nodes: true
  },
  'modified-old-point-on-way': {
    modified: true,
    nodes: true
  },
  'modified-new-line': {
    modified: true,
    ways: true
  },
  'modified-old-relation': {
    modified: true,
    relations: true
  },
  'modified-new-point-tagged': {
    modified: true,
    nodes: true
  },
  'modified-new-point-untagged': {
    modified: true,
    nodes: true
  },
  'modified-new-point-on-way': {
    modified: true,
    nodes: true
  },
  'modified-new-relation': {
    modified: true,
    relations: true
  },
  'deleted-line': {
    deleted: true,
    ways: true
  },
  'deleted-point-tagged': {
    deleted: true,
    nodes: true
  },
  'deleted-point-untagged': {
    deleted: true,
    nodes: true
  },
  'deleted-relation': {
    deleted: true,
    relations: true
  }
};
class Map$1 {
  constructor() {
    this.map = null;
    this.queue = [];
    this.filterLayers = this.filterLayers.bind(this);
    this.relationMembersLayers = {
      'added-relation-member-point': {
        added: true,
        nodes: true,
        changeType: 'added'
      },
      'added-relation-member-point-with-role': {
        added: true,
        nodes: true,
        changeType: 'added',
        extraFilter: ['has', 'role']
      },
      'modified-relation-member-new-point': {
        modified: true,
        nodes: true,
        changeType: 'modifiedNew'
      },
      'modified-relation-member-old-point': {
        modified: true,
        nodes: true,
        changeType: 'modifiedOld'
      },
      'deleted-relation-member-point': {
        deleted: true,
        nodes: true,
        changeType: 'deletedNew'
      },
      'added-relation-member-line': {
        added: true,
        ways: true,
        changeType: 'added'
      },
      'modified-relation-member-new-line': {
        modified: true,
        ways: true,
        changeType: 'modifiedNew'
      },
      'modified-relation-member-old-line': {
        modified: true,
        ways: true,
        changeType: 'modifiedOld'
      },
      'deleted-relation-member-line': {
        deleted: true,
        ways: true,
        changeType: 'deletedNew'
      },
      'modified-relation-member-new-point-on-way': {
        modified: true,
        nodes: true,
        changeType: 'modifiedNew',
        extraFilter: ['==', 'role', 'via']
      },
      'modified-relation-member-old-point-on-way': {
        modified: true,
        nodes: true,
        changeType: 'modifiedOld',
        extraFilter: ['==', 'role', 'via']
      }
    };
    this.layersKey = LAYERS_KEY;
  }

  filterLayers() {
    var selectedActions = [];
    var selectedTypes = [];
    document.querySelectorAll('.cmap-filter-action-section input:checked').forEach(function (checkedElement) {
      selectedActions.push(checkedElement.value);
    });
    document.querySelectorAll('.cmap-filter-type-section input:checked').forEach(function (checkedElement) {
      selectedTypes.push(checkedElement.value);
    });
    var layers = Object.keys(this.layersKey);
    var layersKey = LAYERS_KEY;
    layers.forEach(layer => {
      var isSelectedAction = selectedActions.reduce(function (accum, action) {
        return layersKey[layer][action] || accum;
      }, false);
      var isSelectedType = selectedTypes.reduce(function (accum, type) {
        return layersKey[layer][type] || accum;
      }, false);

      if (isSelectedAction && isSelectedType) {
        this.map.setLayoutProperty(layer, 'visibility', 'visible');
      } else {
        this.map.setLayoutProperty(layer, 'visibility', 'none');
      }

      if (selectedActions.length === 0 || selectedTypes.length === 0) {
        this.map.setLayoutProperty('bg-point', 'visibility', 'none');
        this.map.setLayoutProperty('bg-line', 'visibility', 'none');
      } else {
        this.map.setLayoutProperty('bg-point', 'visibility', 'visible');
        this.map.setLayoutProperty('bg-line', 'visibility', 'visible');
      }
    });
  }

  getMapInstance() {
    return this.map;
  }

  getResult() {
    return this.result;
  }

  remove() {
    if (this.map) {
      this.map.remove();
      this.mapLoaded = false;
      this.map = undefined;
    }
  }

  getRelationMembers(result) {
    let relationMembers = [];
    result.geojson.features.filter(feature => feature.properties.type === 'relation').forEach(relation => relation.properties.relations.forEach(element => {
      element.properties.relation = relation.properties.id;
      element.properties.changeType = relation.properties.changeType;
      relationMembers.push(element);
    })); // exclude invalid relation members coordinates
    // caused by https://github.com/drolbr/Overpass-API/issues/536

    relationMembers = relationMembers.filter(member => member.geometry.coordinates.length > 0);
    relationMembers.forEach(member => {
      member.geometry.coordinates = member.geometry.coordinates.filter(i => typeof i === 'number' || typeof i === 'object' && !isNaN(i[0]));
    });
    return featureCollection(relationMembers);
  }

  addMapSource(result, bounds) {
    if (this.map.getSource('changeset')) {
      this.map.getSource('changeset').setData(result.geojson);
    } else {
      this.map.addSource('changeset', {
        type: 'geojson',
        data: result.geojson
      });
    }

    if (this.map.getSource('relationMembers')) {
      this.map.getSource('relationMembers').setData(this.getRelationMembers(result));
      this.clearRelationMemberHighlight();
    } else {
      this.map.addSource('relationMembers', {
        type: 'geojson',
        data: this.getRelationMembers(result)
      });
    }

    if (this.map.getSource('bbox')) {
      this.map.getSource('bbox').setData(getBoundingBox(bounds));
    } else {
      this.map.addSource('bbox', {
        type: 'geojson',
        data: getBoundingBox(bounds)
      });
    }
  }

  addMapLayers() {
    this.map.addLayer({
      id: 'bbox-line',
      type: 'line',
      source: 'bbox',
      paint: {
        'line-color': '#A58CF2',
        'line-opacity': 0.75,
        'line-width': 2
      }
    });
    this.map.addLayer({
      id: 'bg-line',
      source: 'changeset',
      type: 'line',
      layout: {
        'line-cap': 'round',
        'line-join': 'round'
      },
      paint: {
        'line-color': 'hsl(0, 0%, 15%)',
        'line-width': 12,
        'line-blur': 0.2,
        'line-opacity': {
          base: 1.5,
          stops: [[12, 0.5], [18, 0.2]]
        }
      },
      filter: ['all', ['==', 'type', 'way']]
    });
    this.map.addLayer({
      id: 'bg-point',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': 'hsl(0, 0%, 15%)',
        'circle-blur': 0.2,
        'circle-opacity': {
          base: 1.5,
          stops: [[12, 0.5], [18, 0.2]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 12], [16, 10]]
        }
      },
      filter: ['all', ['==', '$type', 'Point']]
    });
    this.map.addLayer({
      id: 'highlight-line',
      source: 'changeset',
      type: 'line',
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': 'hsl(0, 0%, 75%)',
        'line-width': {
          base: 1,
          stops: [[10, 15], [16, 10]]
        },
        'line-opacity': {
          base: 1.5,
          stops: [[12, 0.75], [18, 0.75]]
        }
      },
      filter: ['all', ['==', 'id', ''], ['==', '$type', 'LineString']]
    });
    this.map.addLayer({
      id: 'highlight-point',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': 'hsl(0, 0%, 75%)',
        'circle-radius': {
          base: 1,
          stops: [[10, 10], [16, 11]]
        },
        'circle-opacity': 0.8
      },
      filter: ['all', ['==', 'id', ''], ['==', '$type', 'Point']]
    });
    this.map.addLayer({
      id: 'highlight-relation-member-line',
      source: 'relationMembers',
      type: 'line',
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': 'hsl(267, 81%, 82%)',
        'line-width': {
          base: 1,
          stops: [[10, 15], [16, 10]]
        },
        'line-opacity': {
          base: 1,
          stops: [[12, 0.5], [18, 0.5]]
        }
      },
      filter: ['all', ['==', 'ref', ''], ['==', '$type', 'LineString']]
    });
    this.map.addLayer({
      id: 'highlight-relation-member-point',
      source: 'relationMembers',
      type: 'circle',
      paint: {
        'circle-color': 'hsl(267, 81%, 82%)',
        'circle-radius': {
          base: 1,
          stops: [[10, 10], [16, 11]]
        },
        'circle-opacity': 0.8
      },
      filter: ['all', ['==', 'ref', ''], ['==', '$type', 'Point']]
    }); // RELATION MEMBERS

    this.map.addLayer({
      id: 'deleted-relation-member-line',
      source: 'relationMembers',
      type: 'line',
      paint: {
        'line-color': '#CC2C47',
        'line-width': {
          base: 1,
          stops: [[8, 3], [12, 5]]
        },
        'line-dasharray': [0.1, 0.25],
        'line-opacity': 0.8
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'modified-relation-member-old-line',
      source: 'relationMembers',
      type: 'line',
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#DB950A',
        'line-width': {
          base: 1,
          stops: [[8, 3], [12, 6]]
        },
        'line-blur': {
          base: 1,
          stops: [[8, 0.25], [12, 0.5]]
        },
        'line-opacity': 0.6
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'modified-relation-member-new-line',
      source: 'relationMembers',
      type: 'line',
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#E8E845',
        'line-width': {
          base: 1,
          stops: [[8, 1], [12, 2]]
        },
        'line-opacity': 0.6
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'added-relation-member-line',
      source: 'relationMembers',
      type: 'line',
      interactive: true,
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#39DBC0',
        'line-width': {
          base: 1,
          stops: [[8, 1.5], [12, 2]]
        },
        'line-opacity': 0.8
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'modified-relation-member-old-point-on-way',
      source: 'relationMembers',
      type: 'circle',
      paint: {
        'circle-color': '#DB950A',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.5]]
        },
        'circle-blur': 0.25,
        'circle-radius': {
          base: 1.5,
          stops: [[10, 2.5], [16, 3.5]]
        }
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'modified-relation-member-new-point-on-way',
      source: 'relationMembers',
      type: 'circle',
      paint: {
        'circle-color': '#E8E845',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.25]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 1.25], [16, 2.25]]
        }
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'deleted-relation-member-point',
      source: 'relationMembers',
      type: 'circle',
      paint: {
        'circle-color': '#CC2C47',
        'circle-radius': {
          base: 1.5,
          stops: [[10, 2], [16, 3]]
        },
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.5]]
        }
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'added-relation-member-point',
      source: 'relationMembers',
      type: 'circle',
      paint: {
        'circle-color': '#39DBC0',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.3], [14, 0.75]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 1.25], [16, 1.9]]
        }
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'added-relation-member-point-with-role',
      source: 'relationMembers',
      type: 'circle',
      paint: {
        'circle-color': '#39DBC0',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.3], [14, 0.75]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 1], [16, 5]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.9,
        'circle-stroke-color': '#39DBC0'
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'modified-relation-member-new-point',
      source: 'relationMembers',
      type: 'circle',
      paint: {
        'circle-color': '#E8E845',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.75]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 2], [16, 7]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.9,
        'circle-stroke-color': '#E8E845'
      },
      filter: ['all', ['==', 'relation', '']]
    });
    this.map.addLayer({
      id: 'modified-relation-member-old-point',
      source: 'relationMembers',
      type: 'circle',
      paint: {
        'circle-color': '#DB950A',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.5]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 1.75], [16, 3]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.9,
        'circle-stroke-color': '#DB950A'
      },
      filter: ['all', ['==', 'relation', '']]
    }); // RELATIONS

    this.map.addLayer({
      id: 'deleted-relation',
      source: 'changeset',
      type: 'line',
      paint: {
        'line-color': '#CC2C47',
        'line-width': {
          base: 1,
          stops: [[8, 1.5], [12, 1.5]]
        },
        'line-dasharray': [0.1, 0.1],
        'line-opacity': 0.8
      },
      filter: ['all', ['==', 'type', 'relation'], ['==', 'changeType', 'deletedNew']]
    });
    this.map.addLayer({
      id: 'modified-old-relation',
      source: 'changeset',
      type: 'line',
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#DB950A',
        'line-width': {
          base: 1,
          stops: [[8, 1.75], [12, 1.75]]
        },
        'line-blur': 0.25,
        'line-opacity': 0.8
      },
      filter: ['all', ['==', 'type', 'relation'], ['==', 'changeType', 'modifiedOld']]
    });
    this.map.addLayer({
      id: 'modified-new-relation',
      source: 'changeset',
      type: 'line',
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#E8E845',
        'line-width': {
          base: 1,
          stops: [[8, 1.25], [12, 1.25]]
        },
        'line-opacity': 0.8
      },
      filter: ['all', ['==', 'type', 'relation'], ['==', 'changeType', 'modifiedNew']]
    });
    this.map.addLayer({
      id: 'added-relation',
      source: 'changeset',
      type: 'line',
      interactive: true,
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#39DBC0',
        'line-width': {
          base: 1,
          stops: [[8, 1], [12, 1]]
        },
        'line-opacity': 0.8
      },
      filter: ['all', ['==', 'type', 'relation'], ['==', 'changeType', 'added']]
    });
    this.map.addLayer({
      id: 'deleted-line',
      source: 'changeset',
      type: 'line',
      paint: {
        'line-color': '#CC2C47',
        'line-width': {
          base: 1,
          stops: [[8, 3], [12, 5]]
        },
        'line-dasharray': [0.1, 0.25],
        'line-opacity': 0.8
      },
      filter: ['all', ['==', 'type', 'way'], ['==', 'changeType', 'deletedNew']]
    });
    this.map.addLayer({
      id: 'modified-old-point-on-way',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#DB950A',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.5]]
        },
        'circle-blur': 0.25,
        'circle-radius': {
          base: 1.5,
          stops: [[10, 2.5], [16, 3.5]]
        }
      },
      filter: ['all', ['==', '$type', 'LineString'], ['==', 'changeType', 'modifiedOld']]
    });
    this.map.addLayer({
      id: 'modified-old-line',
      source: 'changeset',
      type: 'line',
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#DB950A',
        'line-width': {
          base: 1,
          stops: [[8, 3], [12, 6]]
        },
        'line-blur': {
          base: 1,
          stops: [[8, 0.25], [12, 0.5]]
        },
        'line-opacity': 0.6
      },
      filter: ['all', ['==', 'type', 'way'], ['==', 'changeType', 'modifiedOld']]
    });
    this.map.addLayer({
      id: 'modified-new-point-on-way',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#E8E845',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.25]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 1.25], [16, 2.25]]
        }
      },
      filter: ['all', ['==', '$type', 'LineString'], ['==', 'changeType', 'modifiedNew']]
    });
    this.map.addLayer({
      id: 'modified-new-line',
      source: 'changeset',
      type: 'line',
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#E8E845',
        'line-width': {
          base: 1,
          stops: [[8, 1], [12, 2]]
        },
        'line-opacity': 0.6
      },
      filter: ['all', ['==', 'type', 'way'], ['==', 'changeType', 'modifiedNew']]
    });
    this.map.addLayer({
      id: 'added-line',
      source: 'changeset',
      type: 'line',
      interactive: true,
      layout: {
        'line-join': 'round',
        'line-cap': 'round'
      },
      paint: {
        'line-color': '#39DBC0',
        'line-width': {
          base: 1,
          stops: [[8, 1], [12, 1.5]]
        },
        'line-opacity': 0.8
      },
      filter: ['all', ['==', 'type', 'way'], ['==', 'changeType', 'added']]
    });
    this.map.addLayer({
      id: 'deleted-point-untagged',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#CC2C47',
        'circle-radius': {
          base: 1.5,
          stops: [[10, 2], [16, 3]]
        },
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.5]]
        }
      },
      filter: ['all', ['==', 'changeType', 'deletedOld'], ['any', ['==', 'tagsCount', 0], ['==', '$type', 'LineString']]]
    });
    this.map.addLayer({
      id: 'modified-old-point-untagged',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#DB950A',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.5]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 1.75], [16, 3]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.9,
        'circle-stroke-color': '#DB950A'
      },
      filter: ['all', ['==', 'type', 'node'], ['==', 'changeType', 'modifiedOld'], ['==', 'tagsCount', 0]]
    });
    this.map.addLayer({
      id: 'modified-new-point-untagged',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#E8E845',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.5]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 0.75], [16, 2]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.9,
        'circle-stroke-color': '#E8E845'
      },
      filter: ['all', ['==', 'type', 'node'], ['==', 'changeType', 'modifiedNew'], ['==', 'tagsCount', 0]]
    });
    this.map.addLayer({
      id: 'added-point-untagged',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#39DBC0',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.3], [14, 0.75]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 1.25], [16, 1.9]]
        }
      },
      filter: ['all', ['==', 'type', 'node'], ['==', 'changeType', 'added'], ['==', 'tagsCount', 0]]
    });
    this.map.addLayer({
      id: 'deleted-point-tagged',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#CC2C47',
        'circle-radius': {
          base: 1.5,
          stops: [[10, 4], [16, 7]]
        },
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.5]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.75,
        'circle-stroke-color': '#CC2C47'
      },
      filter: ['all', ['==', 'type', 'node'], ['==', 'changeType', 'deletedOld'], ['!=', 'tagsCount', 0]]
    });
    this.map.addLayer({
      id: 'modified-old-point-tagged',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#DB950A',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.75]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 2.5], [16, 9]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.9,
        'circle-stroke-color': '#DB950A'
      },
      filter: ['all', ['==', 'type', 'node'], ['==', 'changeType', 'modifiedOld'], ['!=', 'tagsCount', 0]]
    });
    this.map.addLayer({
      id: 'modified-new-point-tagged',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#E8E845',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.25], [14, 0.75]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 2], [16, 7]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.9,
        'circle-stroke-color': '#E8E845'
      },
      filter: ['all', ['==', 'type', 'node'], ['==', 'changeType', 'modifiedNew'], ['!=', 'tagsCount', 0]]
    });
    this.map.addLayer({
      id: 'added-point-tagged',
      source: 'changeset',
      type: 'circle',
      paint: {
        'circle-color': '#39DBC0',
        'circle-opacity': {
          base: 1.5,
          stops: [[10, 0.3], [14, 0.75]]
        },
        'circle-radius': {
          base: 1.5,
          stops: [[10, 1], [16, 5]]
        },
        'circle-stroke-width': 1,
        'circle-stroke-opacity': 0.9,
        'circle-stroke-color': '#39DBC0'
      },
      filter: ['all', ['==', 'type', 'node'], ['==', 'changeType', 'added'], ['!=', 'tagsCount', 0]]
    });
  }

  renderMap(baseLayer, result) {
    if (!result) {
      if (!this.result) return;
      result = this.result;
    } else {
      this.result = result;
    }

    var bounds = getBounds(result.changeset.bbox);

    if (this.map) {
      if (!this.mapLoaded) {
        this.queue.push([result, bounds]); // TOFIX use variable instead of array

        this.result = result;
        return;
      }

      if (baseLayer && this.oldBaseLayer !== baseLayer) {
        this.map.setStyle(baseLayer);
        this.baseLayerData = [result, bounds];
        this.oldBaseLayer = baseLayer;
      } else {
        this.oldBaseLayer = baseLayer;
        this.addMapSource(result, bounds);
        this.map.fitBounds(bounds, {
          linear: true,
          padding: 200
        });
        this.result = result;
        clearDiff();
      } // why not re attach on('click')
      // if the map is still mounted
      // it will automatically take the latest
      // result thanks to this.result


      return;
    }

    this.map = new mapboxgl.Map({
      container: document.querySelector('.cmap-map'),
      style: baseLayer || 'mapbox://styles/openstreetmap/cjnd8lj0e10i42spfo4nsvoay',
      center: bounds.getCenter(),
      zoom: 14,
      dragRotate: false,
      touchZoomRotate: false
    });

    if (p.touch && !p.mouse) {
      this.map.addControl(new mapboxgl.NavigationControl({
        showCompass: false
      }), 'top-right');
    }

    this.map.on('styledata', () => {
      if (!this.baseLayerData) return;
      console.log('style event fired 2');
      var bounds = this.baseLayerData[1];
      var result = this.baseLayerData[0];
      this.baseLayerData = null;
      this.map.fitBounds(bounds, {
        linear: true,
        padding: 200
      });
      this.addMapSource(result, bounds);
      this.addMapLayers();
      cmap.emit('load');
    });
    this.map.on('load', () => {
      this.mapLoaded = true;

      if (this.queue.length > 0) {
        const index = this.queue.length - 1;
        result = this.queue[index][0];
        bounds = this.queue[index][1];
        this.queue = [];
      }

      this.map.fitBounds(bounds, {
        linear: true,
        padding: 200
      });
      this.addMapSource(result, bounds);
      this.addMapLayers();
      cmap.emit('load'); // if the changeset has only one element, select it when the map is loading

      if (Object.keys(result.featureMap).length === 1) {
        const featureId = Object.keys(result.featureMap)[0];
        this.selectFeature(result.featureMap[featureId][0]);
      }
    });
    this.map.on('click', e => {
      var x1y1 = [e.point.x - 5, e.point.y - 5];
      var x2y2 = [e.point.x + 5, e.point.y + 5];
      var features = this.map.queryRenderedFeatures([x1y1, x2y2], {
        layers: ['added-line', 'added-point-tagged', 'modified-old-line', 'modified-old-point-tagged', 'modified-old-point-untagged', 'modified-new-line', 'modified-new-point-tagged', 'modified-new-point-untagged', 'deleted-line', 'deleted-point-tagged', 'added-relation', 'modified-old-relation', 'modified-new-relation', 'deleted-relation'].concat(Object.keys(this.relationMembersLayers))
      });

      if (features.length) {
        this.selectFeature(features[0]);
      } else {
        this.clearFeature();
      }
    });
  }

  selectFeature(feature) {
    var featureMap = this.result.featureMap;
    var featureId = feature.properties.id;
    var osmType = feature.properties.type;
    this.highlightFeature(featureId);
    displayDiff(featureId, featureMap, document.querySelector('.cmap-diff-metadata'), document.querySelector('.cmap-diff-tags'), document.querySelector('.cmap-diff-members'));
    document.querySelector('.cmap-diff').style.display = 'block';

    if (osmType === 'relation') {
      this.showRelationMembers(featureId);
    }

    cmap.emit('featureChange', osmType, featureId);
  }

  zoomToFeatures(features) {
    this.map.fitBounds(getFeatureBBOX(features), {
      linear: true,
      padding: 200,
      maxZoom: 18
    });
  }

  highlightFeature(featureId) {
    this.map.setFilter('highlight-line', ['==', 'id', featureId]);
    this.map.setFilter('highlight-point', ['==', 'id', featureId]);
  }

  selectMember(feature) {
    this.highlightRelationMemberFeature(feature);
  }

  highlightRelationMemberFeature(featureId) {
    this.map.setFilter('highlight-relation-member-line', ['==', 'ref', featureId]);
    this.map.setFilter('highlight-relation-member-point', ['==', 'ref', featureId]);
  }

  clearHighlight() {
    this.map.setFilter('highlight-line', ['==', 'id', '']);
    this.map.setFilter('highlight-point', ['==', 'id', '']);
    this.clearRelationMemberHighlight();
  }

  clearRelationMemberHighlight() {
    this.map.setFilter('highlight-relation-member-line', ['==', 'ref', '']);
    this.map.setFilter('highlight-relation-member-point', ['==', 'ref', '']);
    Object.keys(this.relationMembersLayers).forEach(layer => this.map.setFilter(layer, ['==', 'relation', '']));
    Object.keys(this.layersKey).filter(layer => !layer.endsWith('relation')).forEach(layer => this.map.setLayoutProperty(layer, 'visibility', 'visible'));
  }

  clearFeature() {
    this.clearHighlight();
    clearDiff();
    cmap.emit('featureChange', null, null);
  }

  showRelationMembers(featureId) {
    Object.keys(this.relationMembersLayers).forEach(layerName => {
      let layer = this.relationMembersLayers[layerName];
      let filter = ['all', ['==', 'relation', featureId], ['==', 'changeType', layer.changeType], ['==', '$type', getLayerType(layer)]];

      if (layer.extraFilter) {
        filter.push(layer.extraFilter);
      }

      this.map.setFilter(layerName, filter);
    });
    Object.keys(this.layersKey).filter(layer => !layer.endsWith('relation')).forEach(layer => this.map.setLayoutProperty(layer, 'visibility', 'none'));
  }

}

function clearDiff() {
  document.querySelector('.cmap-diff').style.display = 'none';
  document.querySelector('.cmap-diff-metadata').innerHTML = '';
  document.querySelector('.cmap-diff-tags').innerHTML = '';
  document.querySelector('.cmap-diff-members').style.display = 'none';
}

function getLayerType(layer) {
  if (layer.nodes) {
    return 'Point';
  }

  if (layer.ways) {
    return 'LineString';
  }

  return null;
}

const cmap = new EventEmitter();
let map;
window.cmap = cmap;
function render(container, changesetId, options) {
  container.style.width = options.width || '1000px';
  container.style.height = options.height || '500px';
  options = options || {};
  options.overpassBase = options.overpassBase || config.overpassBase;
  mapboxgl.accessToken = config.mapboxAccessToken;
  container.classList.add('cmap-loading');

  if (!map) {
    map = new Map$1();
  }

  if (options.data) {
    _render(container, changesetId, options.data, options.disableSidebar);
  } else {
    getChangeset(changesetId, options.overpassBase).then(result => _render(container, changesetId, result)).catch(err => {
      errorMessage(err.msg);
    });
  }

  return cmap;
}
function getMapInstance() {
  return map;
}
function getGL() {
  return mapboxgl;
}

function _render(container, changesetId, result, disableSidebar) {
  renderHTML(container, changesetId, result, disableSidebar);
  container.classList.remove('cmap-loading');
  map.renderMap(false, result);
  var featureMap = result.featureMap;
  cmap.removeAllListeners();
  cmap.on('remove', () => {
    map.remove();
  });
  cmap.on('selectFeature', (geometryType, featureId) => {
    if (geometryType && featureId) {
      map.selectFeature(featureMap[featureId][0], featureMap);
      map.zoomToFeatures(featureMap[featureId]);
    }
  });
  cmap.on('selectMember', featureId => {
    map.selectMember(featureId);
  });
  cmap.on('clearFeature', () => {
    map.clearFeature();
  });
} // Sets initial markup for info box and map container


function renderHTML(container, changesetId, result, disableSidebar) {
  var info;

  if (document.getElementById('seat')) {
    info = document.getElementById('seat');
  } else {
    info = document.createElement('div');
    info.id = 'seat';
    container.appendChild(info);
  }

  container.classList.add('cmap-container'); // Add `tagsCount` to feature properties

  result.geojson.features.forEach(feature => {
    var tags = feature.properties.tags || {};
    feature.properties.tagsCount = Object.keys(tags).length;
  });
  render$1( /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("div", {
    className: "cmap-map"
  }), /*#__PURE__*/React.createElement("div", {
    className: "cmap-diff",
    style: {
      display: 'none'
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: "cmap-diff-metadata cmap-scroll-styled"
  }), /*#__PURE__*/React.createElement("div", {
    className: "cmap-diff-tags cmap-scroll-styled"
  }), /*#__PURE__*/React.createElement("div", {
    className: "cmap-diff-members cmap-scroll-styled"
  })), !disableSidebar && /*#__PURE__*/React.createElement(Sidebar, {
    result: result,
    changesetId: changesetId,
    filterLayers: map.filterLayers,
    toggleLayer: function (e) {
      var layer = e.target.value;

      if (layer === 'satellite') {
        map.renderMap('mapbox://styles/openstreetmap/cjnd8lj0e10i42spfo4nsvoay', result);
      }

      if (layer === 'dark') {
        map.renderMap('mapbox://styles/mapbox/dark-v9', result);
      }

      if (layer === 'streets') {
        map.renderMap('mapbox://styles/mapbox/streets-v9', result);
      }
    }
  })), info);
}

function errorMessage(message) {
  message = message || 'An unexpected error occured';
  document.querySelector('.cmap-info').innerHTML = message;
  document.querySelector('.cmap-sidebar').style.display = 'block';
  document.querySelector('.cmap-layer-selector').style.display = 'none';
  document.querySelector('.cmap-type-selector').style.display = 'none';
}

export { config, getChangeset, getGL, getMapInstance, propsDiff, query, render };
